using System.Text.Json;
using Amazon.Lambda.APIGatewayEvents;
using Amazon.Lambda.Core;
using MediatR;
using Microsoft.Extensions.DependencyInjection;

[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace FunctionUrlWithMediatR;

public class Function
{
    readonly ServiceProvider _serviceProvider;
    
    public Function()
    {
        var services = new ServiceCollection();
        services.AddMediatR(typeof(Function));
        _serviceProvider = services.BuildServiceProvider();
    }
    
    public async Task<APIGatewayHttpApiV2ProxyResponse> FunctionHandler(APIGatewayHttpApiV2ProxyRequest incomingHttpRequest, ILambdaContext context)
    {
        var mediator = _serviceProvider.GetService<IMediator>();
        string textToReturn = incomingHttpRequest.RequestContext.Http.Method switch
        {
            "GET" => await mediator.Send(new GetRequest(incomingHttpRequest)),
            "PUT" => await mediator.Send(new PutRequest(incomingHttpRequest)),
            "POST" => await mediator.Send(new PostRequest(incomingHttpRequest)),
            "DELETE" => await mediator.Send(new DeleteRequest(incomingHttpRequest)),
            _ => default
        };

        var apiResponse = new APIGatewayHttpApiV2ProxyResponse()
        {
            StatusCode = 200,
            Body = JsonSerializer.Serialize(textToReturn)
        };
        return apiResponse;
    }
}
